require "rake_coffee/version"
require "rake"
require "rake/tasklib"
require "rake/clean"

CLOBBER.include("**/**.js").exclude("node_modules/**/*")

module Rake
  class CoffeeTask < TaskLib
    attr_accessor :test_cmd, :test_coverage_cmd,
                  :compile_opts, :lint_opts

    def initialize
      @test_cmd          = "mocha"
      @test_coverage_cmd = "istanbul cover _mocha"
      @lint_opts         = ""
      @compile_opts      = ""
      yield self if block_given?
      define
    end

    def define
      task :default => :test

      coffee_files = ::Rake::FileList["**/*.coffee"]
      js_files     = coffee_files.ext("js")

      desc "compile CoffeeScript files"
      task :compile => js_files

      rule ".js" => ".coffee" do |t|
        sh "coffeelint #{lint_opts} #{t.source}"
        sh "coffee --compile #{compile_opts} #{t.source}"
      end

      desc "run unit test"
      task :test => :compile do
        sh test_cmd
      end

      desc "git commit"
      task :commit => :test do
        sh "git commit -v"
      end

      desc "push to a shared repository"
      task :push => :test do
        sh "git push"
      end

      desc "run unit test with coverage report"
      task :cov => :compile do
        sh test_coverage_cmd
      end
    end
  end
end
